#include <math.h>
#include "input.h"

input::input() : left(P0_14, PullUp), right(P0_11, PullUp), down(P0_12, PullUp), up(P0_13, PullUp), square(P0_16, PullUp), circle(P0_1, PullUp), i2c(P0_5, P0_4)
{
    // Clear all the button states
    for (int i = 0; i < eLastButton; i++)
    {
        m_iPressedTicks[i] = 0;
        m_iHitCount[i] = 0;
    }

    m_fAccelX = 0.0;
    m_fAccelY = 0.0;
    m_fAccelZ = 0.0;
    
    i2c.frequency(400);
    accelWriteRegister(0x2A, 0x01);
}

void input::dobutton(INPUT_BUTTON button, bool bPressed)
{
    // If this button is not being debounced then set the ticks we can debounce the switch
    if (bPressed)
    {
        if (m_iPressedTicks[button] < 1)
        {
            // This is the first time this button was pressed since it was UP
            // Increment the hit count
            m_iHitCount[button]++;
        }
        // Reset the debounce counter
        m_iPressedTicks[button] = BUTTON_DEBOUNCE_TICKS;
    }
    // If this button IS not pressed and we have ticks left... deduct 1 tick
    if (!bPressed && (m_iPressedTicks[button] > 0))
         m_iPressedTicks[button]--;
}

void input::tick()
{
    // Sample all of the buttons
    dobutton(eUp, up.read() == 0);
    dobutton(eDown, down.read() == 0);
    dobutton(eLeft, left.read() == 0);
    dobutton(eRight, right.read() == 0);
    dobutton(eCircle, circle.read() == 0);
    dobutton(eSquare, square.read() == 0);
    
    // UGHHHHHHH - accelReadRegisters() is BALLS SLOW!!! Disable for NOW!
    /*
    // Sample the accelerometers
    char buffer[6];
    accelReadRegisters(0x01, buffer, 6);
    m_fAccelX = accelConvert(buffer);
    m_fAccelY = accelConvert(buffer + 2);
    m_fAccelZ = accelConvert(buffer + 4);
    */
}

void input::clear()
{
    for (int i = 0; i < eLastButton; i++)
    {
        m_iHitCount[i] = 0;
        m_iPressedTicks[i] = 0;
    }
}

int input::hit(INPUT_BUTTON button)
{
    if (button >= eLastButton) return 0;
    // Store the hit count for this button
    int hit_count = m_iHitCount[button];
    // Clear the hit count
    m_iHitCount[button] = 0;
    // Return the stored hit count
    return hit_count;
}
 
bool input::pressed(INPUT_BUTTON button)
{
    if (button >= eLastButton) return false;
    
    return (m_iPressedTicks[button] > 0);
}


int input::accel_x()
{
    return (int)(m_fAccelX * 100.0);
}

int input::accel_y()
{
    return (int)(m_fAccelY * 100.0);
}

int input::accel_z()
{
    return (int)(m_fAccelZ * 100.0);
}

int input::accel_roll()
{
    // Roll and Pitch are swapped on the Retro... 
    return (int)((atan(m_fAccelX / sqrt((m_fAccelY * m_fAccelY) + (m_fAccelZ * m_fAccelZ))) * 180.0) / 3.14159);
}

int input::accel_pitch()
{
    // Roll and Pitch are swapped on the Retro.. (Pitch is Roll, Roll is Pitch)
    return (int)((atan(m_fAccelY / sqrt((m_fAccelX * m_fAccelX) + (m_fAccelZ * m_fAccelZ))) * 180.0) / 3.14159);
}

void input::accelReadRegisters(char address, char* buffer, int len)
{
    // BALLS SLOW!!!
    i2c.write(I2C_ADDR, &address, 1, true);
    i2c.read(I2C_ADDR | 1, buffer, len);
}

int input::accelWriteRegister(char address, char value)
{    
    char buffer[2] = { address, value };    
    return i2c.write(I2C_ADDR, buffer, 2);
}

float input::accelConvert(char* buffer)
{
    float val = ((buffer[0] << 2) | (buffer[1] >> 6));           
    if (val > 511.0) val -= 1024.0;
    return val / 512.0;
}
